#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import argparse
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
import time
from getpass import getpass

from naruno.accounts.get_balance import GetBalance
from naruno.blockchain.block.create_block import CreateBlock
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.config import MY_TRANSACTION_EXPORT_PATH
from naruno.consensus.consensus_main import consensus_trigger
from naruno.lib.backup.naruno_export import naruno_export
from naruno.lib.backup.naruno_import import naruno_import
from naruno.lib.export import export_the_transactions
from naruno.lib.log import get_logger
from naruno.lib.mix.mixlib import (banner_maker, menu_maker, menu_space,
                                   question_maker, quit_menu_maker)
from naruno.lib.perpetualtimer import perpetualTimer
from naruno.lib.safety import safety_check
from naruno.lib.settings_system import (baklava_settings, d_mode_settings,
                                        publisher_mode_settings,
                                        t_mode_settings, the_settings)
from naruno.lib.sign import sign
from naruno.lib.status import Status
from naruno.lib.verify import verify
from naruno.node.server.server import server
from naruno.node.unl import Unl
from naruno.transactions.my_transactions.check_proof import CheckProof
from naruno.transactions.my_transactions.get_proof import GetProof
from naruno.transactions.my_transactions.save_to_my_transaction import \
    SavetoMyTransaction
from naruno.transactions.print_transactions import PrintTransactions
from naruno.transactions.send import send
from naruno.wallet.delete_current_wallet import delete_current_wallet
from naruno.wallet.print_wallets import print_wallets
from naruno.wallet.wallet_create import wallet_create
from naruno.wallet.wallet_import import wallet_import
from naruno.wallet.wallet_selector import wallet_selector

logger = get_logger("CLI")


def show_menu():
    """
    Prints some information and the menu.
    """

    print(
        banner_maker(
            sc_name="Naruno",
            description=
            "Naruno is a lightning-fast, secure, and scalable blockchain that is able to create transaction proofs and verification via raw data and timestamp. We remove the archive nodes and lazy web3 integrations. With Naruno everyone can get the proof (5-10MB) of their transactions via their nodes and after everyone can use in another node for verification the raw data and timestamp. Also you can integrate your web3 applications with 4 code lines (just python for now) via our remote app system.",
            author="Naruno Developers",
        ))

    print(
        menu_space() +
        menu_maker(menu_number="pw", menu_text="Print Wallets") +
        menu_maker(menu_number="w", menu_text="Change Wallet") +
        menu_maker(menu_number="cw", menu_text="Create Wallet") +
        menu_maker(menu_number="dw", menu_text="Delete Wallet") +
        menu_space() + menu_maker(menu_number="sc", menu_text="Send Coin") +
        menu_maker(menu_number="scd", menu_text="Send Coin Data") +
        menu_space() + menu_maker(menu_number="gb", menu_text="Get Balance") +
        menu_space() +
        menu_maker(menu_number="ndstart", menu_text="Node Start") +
        menu_maker(menu_number="ndstop", menu_text="Node Stop") +
        menu_maker(menu_number="ndconnect", menu_text="Node Connect") +
        menu_maker(menu_number="ndconnectmixdb",
                   menu_text="Node Connect from mixdb") +
        menu_maker(menu_number="ndnewunl", menu_text="Add new UNL node") +
        menu_maker(menu_number="ndid", menu_text="Print my id") +
        menu_space() +
        menu_maker(menu_number="testmodeon", menu_text="Test mode ON") +
        menu_maker(menu_number="testmodeoff", menu_text="Test mode OF") +
        menu_maker(menu_number="debugmodeon", menu_text="Debug mode ON") +
        menu_maker(menu_number="debugmodeoff", menu_text="Debug mode OF") +
        menu_space() + menu_maker(menu_number="exptrcsv",
                                  menu_text="Export Transaction as CSV") +
        menu_maker(menu_number="returntrs",
                   menu_text="Export Transaction as CSV") + menu_space() +
        menu_maker(menu_number="status", menu_text="Prints the status") +
        menu_space() + menu_maker(menu_number="getblock",
                                  menu_text="Get block From Other Nodes") +
        menu_space() +
        menu_maker(menu_number="getproof", menu_text="Get Proof") +
        menu_maker(menu_number="checkproof", menu_text="Check Proof") +
        menu_space() +
        menu_maker(menu_number="narunoexport", menu_text="Export backup") +
        menu_maker(menu_number="narunoimport", menu_text="Import backup") +
        menu_space() +
        menu_maker(menu_number="sign", menu_text="Sign and export an data") +
        menu_maker(menu_number="verify", menu_text="Verify the signed data") +
        menu_space() + menu_maker(menu_number="publishermodeon",
                                  menu_text="Publisher Mode On") +
        menu_maker(menu_number="publishermodeoff",
                   menu_text="Publisher Mode Off") + menu_space())

    print(quit_menu_maker(mode="main"))


def menu():
    """
    The main structure of the cli mode, this function prints the menu,
    listens to the entries, makes the directions.
    """

    animation = [
        "[■□□□□□□□□□]",
        "[■■□□□□□□□□]",
        "[■■■□□□□□□□]",
        "[■■■■□□□□□□]",
        "[■■■■■□□□□□]",
        "[■■■■■■□□□□]",
        "[■■■■■■■□□□]",
        "[■■■■■■■■□□]",
        "[■■■■■■■■■□]",
        "[■■■■■■■■■■]",
    ]

    for item in animation:
        time.sleep(0.1)
        sys.stdout.write("\r" + item)
        sys.stdout.flush()

    while True:
        show_menu()
        choices_input = question_maker(mode="main")

        if choices_input == "pw":
            print_wallets()

        if choices_input == "w":
            wallet_selector(input("Please select wallet: "))

        if choices_input == "cw":
            wallet_create(getpass("Password: "))

        if choices_input == "dw" and input("Are you sure ? (y or n): ") == "y":
            delete_current_wallet()
        if choices_input == "sc":
            block = GetBlock()
            send_tx = send(
                getpass("Password: "),
                input("Please write receiver adress: "),
                amount=input("Coin Amount (ex. 1.0): "),
                block=block,
            )
            if send_tx != False:
                SavetoMyTransaction(send_tx, sended=True)
                server.send_transaction(send_tx)
                SaveBlock(block)
        if choices_input == "scd":
            block = GetBlock()
            send_tx = send(
                getpass("Password: "),
                input("Please write receiver adress: "),
                amount=input("Coin Amount (ex. 1.0): "),
                data=input("Data: "),
                block=block,
            )
            if send_tx != False:
                SavetoMyTransaction(send_tx, sended=True)
                server.send_transaction(send_tx)
                SaveBlock(block)
        if choices_input == "gb":
            print(GetBalance(wallet_import(-1, 0)))
        if choices_input == "help":
            show_menu()
        if choices_input == "ndstart":
            server(str(input("ip: ")), int(input("port: ")))
        if choices_input == "ndstop":
            server.Server.stop()
        if choices_input == "ndconnect":
            server.Server.connect(str(input("node ip: ")),
                                  int(input("node port: ")))

        if choices_input == "ndconnectmixdb":
            server.connectionfrommixdb()
        if choices_input == "ndnewunl":
            Unl.save_new_unl_node(input("Please write ID of the node: "))
        if choices_input == "ndid":
            print(server.id)
        if choices_input == "testmodeon":
            t_mode_settings(True)
        if choices_input == "testmodeoff":
            t_mode_settings(False)
        if choices_input == "debugmodeon":
            d_mode_settings(True)
        if choices_input == "debugmodeoff":
            d_mode_settings(False)

        if choices_input == "publishermodeon":
            publisher_mode_settings(True)
        if choices_input == "publishermodeoff":
            publisher_mode_settings(False)

        if choices_input == "exptrcsv":
            if export_the_transactions():
                print(
                    f"CSV file created in {MY_TRANSACTION_EXPORT_PATH} directory"
                )
            else:
                print("You have not a transaction")

        if choices_input == "returntrs":
            PrintTransactions()

        if choices_input == "getblock":
            if the_settings()["test_mode"]:
                the_block = CreateBlock()
                SaveBlock(the_block)
                server.Server.send_block_to_other_nodes()
                logger.info("Consensus timer is started")
                perpetualTimer(the_block.consensus_timer, consensus_trigger)
            else:
                server.Server.send_me_full_block()

        if choices_input == "status":
            print(Status())

        if choices_input == "getproof":
            print(GetProof(input("Please write the transaction signature: ")))
        if choices_input == "checkproof":
            print(CheckProof(input("Please write the path of proof: ")))

        if choices_input == "narunoexport":
            print(naruno_export())
        if choices_input == "narunoimport":
            print(
                naruno_import(
                    input("Please write the path of exported backup: ")))

        if choices_input == "sign":
            print(sign(input("Please write the data: "),
                       getpass("Password: ")))

        if choices_input == "verify":
            print(verify(input("Please write the signed data path: ")))

        input("Press enter to continue (screen will be cleaned)...")
        os.system("cls" if os.name == "nt" else "clear")

        if choices_input == "0":
            exit()


def arguments():
    """
    This function parses the arguments and makes the directions.
    """

    parser = argparse.ArgumentParser(
        description=
        "Naruno is a lightning-fast, secure, and scalable blockchain that is able to create transaction proofs and verification via raw data and timestamp. We remove the archive nodes and lazy web3 integrations. With Naruno everyone can get the proof (5-10MB) of their transactions via their nodes and after everyone can use in another node for verification the raw data and timestamp. Also you can integrate your web3 applications with 4 code lines (just python for now) via our remote app system. Use the menu (-m) or GUI to gain full control and use the node, operation, etc."
    )

    parser.add_argument("-pw",
                        "--printwallet",
                        action="store_true",
                        help="Print Wallets")

    parser.add_argument("-w", "--wallet", type=int, help="Change Wallet")

    parser.add_argument("-cw", "--createwallet", help="Create wallet")

    parser.add_argument("-dw",
                        "--deletewallet",
                        action="store_true",
                        help="Delete wallet")

    parser.add_argument("-gb",
                        "--getbalance",
                        action="store_true",
                        help="Get Balance")

    parser.add_argument("-ndnunl",
                        "--ndnewunl",
                        type=str,
                        help="Add new UNL node")

    parser.add_argument("-ndid",
                        "--ndid",
                        action="store_true",
                        help="Print my id")

    parser.add_argument("-tmon",
                        "--testmodeon",
                        action="store_true",
                        help="Test Mode On")
    parser.add_argument("-tmoff",
                        "--testmodeoff",
                        action="store_true",
                        help="Test Mode Off")

    parser.add_argument("-dmon",
                        "--debugmodeon",
                        action="store_true",
                        help="Debug Mode On")
    parser.add_argument("-dmoff",
                        "--debugmodeoff",
                        action="store_true",
                        help="Debug Mode Off")

    parser.add_argument("-pmon",
                        "--publishermodeon",
                        action="store_true",
                        help="Publisher Mode On")
    parser.add_argument("-pmoff",
                        "--publishermodeoff",
                        action="store_true",
                        help="Publisher Mode Off")

    parser.add_argument("-bon",
                        "--baklavaon",
                        action="store_true",
                        help="Baklava On")
    parser.add_argument("-boff",
                        "--baklavaoff",
                        action="store_true",
                        help="Baklava Off")

    parser.add_argument(
        "-exptrcsv",
        "--exporttransactioncsv",
        action="store_true",
        help="Exports the transaction as csv",
    )

    parser.add_argument(
        "-returntrans",
        "--returntransactions",
        action="store_true",
        help="Exports the transaction as csv",
    )

    parser.add_argument("-st",
                        "--status",
                        action="store_true",
                        help="Exports the transaction as csv")

    parser.add_argument("-gp",
                        "--getproof",
                        type=str,
                        help="Get proof of given transaction signature")

    parser.add_argument("-cp",
                        "--checkproof",
                        type=str,
                        help="Checks the given proof")

    parser.add_argument("-narunoexport",
                        "--narunoexport",
                        action="store_true",
                        help="Export backup")

    parser.add_argument("-narunoimport",
                        "--narunoimport",
                        type=str,
                        help="Import backup")

    parser.add_argument(
        "-m",
        "--menu",
        action="store_true",
        help="An optional boolean for open the menu.",
    )

    parser.add_argument(
        "-i",
        "--interface",
        type=str,
        help="Interface",
    )

    parser.add_argument(
        "-t",
        "--timeout",
        type=int,
        help="Timeout",
    )

    args = parser.parse_args()

    if len(sys.argv) < 2:
        parser.print_help()

    if args.printwallet:
        print_wallets()

    if args.getbalance:
        print(GetBalance(wallet_import(-1, 0)))

    if args.ndid:
        print(server.id)

    if args.testmodeon:
        t_mode_settings(True)
    if args.testmodeoff:
        t_mode_settings(False)
    if args.debugmodeon:
        d_mode_settings(True)
    if args.debugmodeoff:
        d_mode_settings(False)

    if args.publishermodeon:
        publisher_mode_settings(True)
    if args.publishermodeoff:
        publisher_mode_settings(False)

    if args.baklavaon:
        baklava_settings(True)
    if args.baklavaoff:
        baklava_settings(False)

    if args.exporttransactioncsv:
        if export_the_transactions():
            print(
                f"CSV file created in {MY_TRANSACTION_EXPORT_PATH} directory")
        else:
            print("You have not a transaction")

    if args.returntransactions:
        PrintTransactions()

    if args.status:
        print(Status())

    safety_check(args.interface, args.timeout)

    if args.ndnewunl is not None:
        Unl.save_new_unl_node(args.ndnewunl)

    if args.createwallet is not None:
        wallet_create(args.createwallet)

    if args.deletewallet:
        delete_current_wallet()

    if args.wallet is not None:
        wallet_selector(args.wallet)

    if args.getproof is not None:
        print(GetProof(args.getproof))

    if args.checkproof is not None:
        print(CheckProof(args.checkproof))

    if args.narunoexport:
        print(naruno_export())

    if args.narunoimport is not None:
        naruno_import(args.narunoimport)

    if args.menu:
        menu()


def start():
    """
    Start the CLI mode with arguments.
    """

    logger.info("Starting CLI mode")

    arguments()


if __name__ == "__main__":
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import threading

from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.finished.finished_main import finished_main
from naruno.consensus.ongoing.ongoing_main import ongoing_main
from naruno.lib.log import get_logger
from naruno.lib.perpetualtimer import perpetualTimer
from naruno.node.client.client import client
from naruno.node.server.server import server
from naruno.transactions.cleaner import Cleaner
from naruno.transactions.pending.get_pending import GetPending

from naruno.consensus.sync.sync import sync

logger = get_logger("CONSENSUS")


def consensus_trigger(
    custom_block: Block = None,
    custom_candidate_class: candidate_block = None,
    custom_unl_nodes: dict = None,
    custom_UNL_NODES_PATH: str = None,
    custom_server: server = None,
    custom_unl: client = None,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_BLOCKS_PATH: str = None,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,
    pass_sync: bool = False,
    dont_clean=False,
) -> Block:
    """
    Consensus process consists of 2 stages. This function makes
    the necessary redirects according to the situation and works
    to shorten the block time.
    """

    block = (GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
             if custom_block is None else custom_block)
    pending_list_txs = GetPending()

    logger.info(
        f"BLOCK#{block.sequence_number}:{block.empty_block_number} Consensus process started"
    )

    logger.info("Consensus Sync process started")
    threading.Thread(
        target=sync,
        args=(
            block,
            pending_list_txs,
            custom_server,
        ),
    ).start()

    if block.validated:
        logger.info("BLOCK is an validated block, consensus process is finished")
        finished_main(
            block,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_BLOCKS_PATH=custom_BLOCKS_PATH,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
            pass_sync=pass_sync,
            dont_clean=dont_clean
        )
    else:
        logger.info("BLOCK is an unvalidated block, consensus process is ongoing")
        ongoing_main(
            block,
            custom_candidate_class=custom_candidate_class,
            custom_unl_nodes=custom_unl_nodes,
            custom_UNL_NODES_PATH=custom_UNL_NODES_PATH,
            custom_server=custom_server,
            custom_unl=custom_unl,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
            pass_sync=pass_sync,
        )


    logger.info("Consensus process is done")
    return block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import sys

from speed_calculator import calculate

sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", ".."))

from naruno.accounts.account import Account
from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import (GetBlockshash,
                                                           GetBlockshash_part,
                                                           SaveBlockshash,
                                                           SaveBlockshash_part)
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.hash.accounts_hash import AccountsHash
from naruno.blockchain.block.save_block import SaveBlock
from naruno.lib.mix.merkle_root import MerkleTree
from naruno.transactions.check.check_transaction import \
    CheckTransaction
from naruno.transactions.transaction import Transaction


class Transactions_IO_Performance_Analyzer:
    """
    This class is used to analyze the performance of GetBlock
    """

    def __init__(self):
        self.block = Block("test")

        the_transaction_json = {
            "sequence_number": 1,
            "signature":
            "MEUCIHABt7ypkpvFlpqL4SuogwVuzMu2gGynVkrSw6ohZ/GyAiEAg2O3iOei1Ft/vQRpboX7Sm1OOey8a3a67wPJaH/FmVE=",
            "fromUser":
            "MFYwEAYHKoZIzj0CAQYFK4EEAAoDQgAE0AYA7B+neqfUA17wKh3OxC67K8UlIskMm9T2qAR+pl+kKX1SleqqvLPM5bGykZ8tqq4RGtAcGtrtvEBrB9DTPg==",
            "toUser": "onur",
            "data": "blockchain-lab",
            "amount": 5000.0,
            "transaction_fee": 0.02,
            "transaction_time": 1656764224,
        }
        the_transaction = Transaction.load_json(the_transaction_json)
        self.block = Block(the_transaction.fromUser)
        self.block.transaction_delay_time = 60
        self.block.minumum_transfer_amount = 1000

        self.the_transaction_list = [
            the_transaction for i in range(self.block.max_tx_number)
        ]

    def analyze(self) -> float:
        """
        This function is used to analyze the performance of GetBlock
        """

        result = (
            calculate(self.save_operation)[0],
            calculate(self.get_operation)[0],
            os.path.getsize("db/Block_Performance_Analyzer_block.pf") /
            1000000,
        )

        os.remove("db/Block_Performance_Analyzer_block.pf")

        return result

    def save_operation(self):
        """
        This function is used to analyze the performance of GetBlock
        """

        SaveBlock(
            self.block,
            custom_TEMP_BLOCK_PATH="db/Block_Performance_Analyzer_block.pf",
        )

        for the_transaction in self.the_transaction_list:
            CheckTransaction(
                self.block,
                the_transaction,
                custom_current_time=(the_transaction.transaction_time + 5),
                custom_sequence_number=0,
                custom_balance=100000,
            )

    def get_operation(self):
        """
        This function is used to analyze the performance of GetBlock
        """

        GetBlock("db/Block_Performance_Analyzer_block.pf")


if __name__ == "__main__":
    print(Transactions_IO_Performance_Analyzer().analyze())



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.rounds.round_1.checks.candidate_blocks.candidate_blocks_main import \
    candidate_blocks_check
from naruno.consensus.rounds.round_1.checks.time.time_difference.time_difference_main import \
    time_difference_check
from naruno.lib.log import get_logger

logger = get_logger("CONSENSUS_FIRST_ROUND")


def round_check(
    block: Block, 
    candidate_class: candidate_block,
    unl_nodes: dict,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,    
    ) -> bool:
    logger.info("Round 1 checking is started")
    if not time_difference_check(
        block=block,
        custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
        custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
        custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
        custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,           
        ):
        return False    
    if not candidate_blocks_check(candidate_class, unl_nodes):
        return False
    logger.info("Round 1 checking is True")
    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.


import base64


def encrypt(clear, key):
    """
    Encrypt the string and return the encoded cipher.
    """

    enc = []
    for i in range(len(clear)):
        key_c = key[i % len(key)]
        enc_c = chr((ord(clear[i]) + ord(key_c)) % 256)
        enc.append(enc_c)
    return base64.urlsafe_b64encode("".join(enc).encode()).decode()


def decrypt(enc, key):
    """
    Decrypt the encoded cipher and return the clear string.
    """

    dec = []
    enc = base64.urlsafe_b64decode(enc).decode()
    for i in range(len(enc)):
        key_c = key[i % len(key)]
        dec_c = chr((256 + ord(enc[i]) - ord(key_c)) % 256)
        dec.append(dec_c)
    return "".join(dec)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time
from hashlib import sha256

from naruno.accounts.get_balance import GetBalance
from naruno.accounts.get_sequence_number import GetSequanceNumber
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.get_block import GetBlock
from naruno.lib.log import get_logger
from naruno.lib.settings_system import the_settings
from naruno.transactions.get_transaction import GetTransaction
from naruno.transactions.transaction import Transaction
from naruno.wallet.ellipticcurve.ecdsa import Ecdsa
from naruno.wallet.ellipticcurve.privateKey import PrivateKey
from naruno.wallet.wallet_import import wallet_import


from urllib.request import urlopen
from urllib import request, parse


logger = get_logger("TRANSACTIONS")


def send(
    password,
    to_user,
    amount=None,
    data="",
    block=None,
    custom_current_time=None,
    custom_sequence_number=None,
    custom_balance=None,
    custom_account_list=None,
    custom_set_sequence_number=None,
):
    """
    The main function for sending the transaction.

    Inputs:
        password: The password of the wallet.
        to_user: The address of the recipient.
        amount: The amount of the transaction.
        data: The data of the transaction.

    """
    if (wallet_import(int(the_settings()["wallet"]),
                      2) == sha256(password.encode("utf-8")).hexdigest()):

        my_private_key = wallet_import(-1, 1, password)
        my_public_key = "".join([
            l.strip() for l in wallet_import(-1, 0).splitlines()
            if l and not l.startswith("-----")
        ])    
        if not the_settings()["baklava"]:
            block = block if block is not None else GetBlock()

            minumum_transfer_amount = block.minumum_transfer_amount
            max_data_size = block.max_data_size
            max_tx_number = block.max_tx_number
            transaction_fee = block.transaction_fee

            the_balance = GetBalance(to_user,
                        account_list=custom_account_list,
                        dont_convert=True,
                        block=block)
            sequence_number = GetSequanceNumber(my_public_key) + 1
        else:
            the_balance = float(urlopen(f"http://test_net.1.naruno.org:8000/balance/get/?address={to_user}").read().decode("utf-8").replace("\n", ""))
            sequence_number = float(urlopen(f"http://test_net.1.naruno.org:8000/sequence/get/?address={wallet_import(-1,3)}").read().decode("utf-8").replace("\n", "")) + 1


            transaction_fee = float(urlopen("http://test_net.1.naruno.org:8000/blocktransactionfee/get/").read().decode("utf-8"))
            max_tx_number = int(urlopen("http://test_net.1.naruno.org:8000/blockmaxtxnumber/get/").read().decode("utf-8"))
            max_data_size = int(urlopen("http://test_net.1.naruno.org:8000/blockmaxdatasize/get/").read().decode("utf-8"))
            minumum_transfer_amount = int(urlopen("http://test_net.1.naruno.org:8000/blockminumumtransferamount/get/").read().decode("utf-8"))

        if custom_set_sequence_number is not None:
            sequence_number = custom_set_sequence_number

        the_minumum_amount = 0
        if (the_balance >= 0):
            pass
        else:
            the_minumum_amount = minumum_transfer_amount
        amount = amount if amount is not None else the_minumum_amount

        try:
            amount = float(amount)
        except ValueError:
            logger.exception("This is not float coin amount.")
            return False

        if amount < 0:
            logger.error("This is negative coin amount.")
            return False

        if (max_data_size / max_tx_number) < len(data):
            logger.error("The data is too long.")
            return False

        decimal_amount = len(str(transaction_fee).split(".")[1])
        if len(str(amount).split(".")[1]) > decimal_amount:
            logger.error(
                f"The amount of decimal places is more than {decimal_amount}.")
            return False




        # Get the current fee
        transaction_fee = transaction_fee

        tx_time = int(time.time())
        the_transaction = Transaction(
            sequence_number,
            "signature",
            my_public_key,
            to_user,
            data,
            amount,
            transaction_fee,
            tx_time,
        )
        the_transaction.signature = Ecdsa.sign(
                        (str(the_transaction.sequence_number) + the_transaction.fromUser + str(the_transaction.toUser) +
                        str(the_transaction.data)) + str(the_transaction.amount) + str(the_transaction.transaction_fee) +
                        str(the_transaction.transaction_time),
                        PrivateKey.fromPem(my_private_key),
                    ).toBase64()        
        logger.info(f"Transaction: {the_transaction.dump_json()}")

        sending_result = False

        if not the_settings()["baklava"]:
            sending_result = GetTransaction(
                    block,
                    the_transaction,
                    custom_current_time=custom_current_time,
                    custom_sequence_number=custom_sequence_number,
                    custom_balance=custom_balance,
                    custom_account_list=custom_account_list,
            )
        else:
            logger.info("Sending the transaction to the baklava network.")
            logger.info(f"Transaction: {the_transaction.dump_json()}")
            the_data = {
                "sequence_number": the_transaction.sequence_number,
                "signature": the_transaction.signature,
                "fromUser": the_transaction.fromUser,
                "toUser": the_transaction.toUser,
                "amount": the_transaction.amount,
                "data": the_transaction.data,
                "transaction_fee": the_transaction.transaction_fee,
                "time_of_transaction": the_transaction.transaction_time,
            }

            data = parse.urlencode(the_data).encode()
            req =  request.Request("http://test_net.1.naruno.org:8000/transaction/send/", data=data) # this will make the method "POST"
            try:
                resp = request.urlopen(req)
                sending_result = True
            except:
                sending_result = False

        if sending_result:

            del my_private_key
            del password

            return the_transaction
        else:
            logger.error("The transaction is not valid.")
            return False

    else:
        logger.error("Password is not correct")
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from naruno.wallet.ellipticcurve.curve import getCurveByOid
from naruno.wallet.ellipticcurve.curve import secp256k1
from naruno.wallet.ellipticcurve.math import Math
from naruno.wallet.ellipticcurve.publicKey import PublicKey
from naruno.wallet.ellipticcurve.utils.binary import base64FromByteString
from naruno.wallet.ellipticcurve.utils.binary import byteStringFromBase64
from naruno.wallet.ellipticcurve.utils.binary import byteStringFromHex
from naruno.wallet.ellipticcurve.utils.binary import hexFromByteString
from naruno.wallet.ellipticcurve.utils.binary import intFromHex
from naruno.wallet.ellipticcurve.utils.der import DerFieldType
from naruno.wallet.ellipticcurve.utils.der import encodeConstructed
from naruno.wallet.ellipticcurve.utils.der import encodePrimitive
from naruno.wallet.ellipticcurve.utils.der import hexFromInt
from naruno.wallet.ellipticcurve.utils.der import parse
from naruno.wallet.ellipticcurve.utils.integer import RandomInteger
from naruno.wallet.ellipticcurve.utils.pem import createPem
from naruno.wallet.ellipticcurve.utils.pem import getPemContent


class PrivateKey:

    def __init__(self, curve=secp256k1, secret=None):
        self.curve = curve
        self.secret = secret or RandomInteger.between(1, curve.N - 1)

    def publicKey(self):
        curve = self.curve
        publicPoint = Math.multiply(
            p=curve.G,
            n=self.secret,
            N=curve.N,
            A=curve.A,
            P=curve.P,
        )
        return PublicKey(point=publicPoint, curve=curve)

    def toDer(self):
        publicKeyString = self.publicKey().toString(encoded=True)
        hexadecimal = encodeConstructed(
            encodePrimitive(DerFieldType.integer, 1),
            encodePrimitive(DerFieldType.octetString, hexFromInt(self.secret)),
            encodePrimitive(
                DerFieldType.oidContainer,
                encodePrimitive(DerFieldType.object, self.curve.oid),
            ),
            encodePrimitive(
                DerFieldType.publicKeyPointContainer,
                encodePrimitive(DerFieldType.bitString, publicKeyString),
            ),
        )
        return byteStringFromHex(hexadecimal)

    def toPem(self):
        der = self.toDer()
        return createPem(content=base64FromByteString(der),
                         template=_pemTemplate)

    @classmethod
    def fromPem(cls, string):
        privateKeyPem = getPemContent(pem=string, template=_pemTemplate)
        return cls.fromDer(byteStringFromBase64(privateKeyPem))

    @classmethod
    def fromDer(cls, string):
        hexadecimal = hexFromByteString(string)
        privateKeyFlag, secretHex, curveData, publicKeyString = parse(
            hexadecimal)[0]

        curve = getCurveByOid(curveData[0])
        privateKey = cls.fromString(string=secretHex, curve=curve)

        return privateKey

    @classmethod
    def fromString(cls, string, curve=secp256k1):
        return PrivateKey(secret=intFromHex(string), curve=curve)


_pemTemplate = """{content}"""



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.lib.log import get_logger
from naruno.lib.settings_system import change_wallet
from naruno.lib.settings_system import the_settings
from naruno.wallet.get_saved_wallet import get_saved_wallet
from naruno.wallet.wallet_delete import wallet_delete
from naruno.wallet.wallet_import import wallet_import

logger = get_logger("WALLET")


def delete_current_wallet():
    """
    Deletes the current wallet.
    """

    if the_settings()["wallet"] != 0:
        saved_wallets = get_saved_wallet()
        selected_wallet_pubkey = wallet_import(int(the_settings()["wallet"]),
                                               0)
        for each_wallet in saved_wallets:
            if selected_wallet_pubkey == saved_wallets[each_wallet][
                    "publickey"]:
                change_wallet(0)
                if wallet_delete(each_wallet):
                    logger.info(f"Wallet deleted: {each_wallet}")
                    return True
    else:
        logger.info("First wallet cannot be deleted.")
        return False
